@@ -1,4 +1,4 @@
-# $Id: Gtk.pm,v 1.16 2002/05/29 05:24:10 rcaputo Exp $
+# $Id: Gtk.pm,v 1.17 2002/07/16 17:40:15 rcaputo Exp $
 
 # Gtk-Perl substrate for POE::Kernel.
 
@@ -8,7 +8,7 @@ package POE::Kernel::Gtk;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.16 $ ))[1];
+$VERSION = (qw($Revision: 1.17 $ ))[1];
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -17,7 +17,7 @@ use POE::Preprocessor;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.16 $ ))[1];
+$VERSION = (qw($Revision: 1.17 $ ))[1];
 
 # Ensure that no other substrate module has been loaded.
 BEGIN {
@@ -128,7 +128,7 @@ macro substrate_resume_watching_child_signals () {
 ### Time.
 
 macro substrate_resume_time_watcher {
-  my $next_time = ($kr_events[0]->[ST_TIME] - time()) * 1000;
+  my $next_time = ($poe_kernel->[KR_EVENTS]->[0]->[ST_TIME] - time()) * 1000;
   $next_time = 0 if $next_time < 0;
   $poe_kernel->[KR_WATCHER_TIMER] =
     Gtk->timeout_add( $next_time, \&_substrate_event_callback );
@@ -141,6 +141,11 @@ macro substrate_reset_time_watcher {
   {% substrate_resume_time_watcher %}
 }
 
+sub _substrate_resume_timer {
+  Gtk->idle_remove($poe_kernel->[KR_WATCHER_TIMER]);
+  {% substrate_resume_time_watcher %}
+}
+
 macro substrate_pause_time_watcher {
   # does nothing
 }
@@ -233,10 +238,7 @@ macro substrate_define_callbacks {
 
     # Register the next timeout if there are events left.
     if (@kr_events) {
-      my $next_time = ($kr_events[0]->[ST_TIME] - time()) * 1000;
-      $next_time = 0 if $next_time < 0;
-      $self->[KR_WATCHER_TIMER] =
-        Gtk->timeout_add( $next_time, \&_substrate_event_callback );
+      $self->[KR_WATCHER_TIMER] = Gtk->idle_add(\&_substrate_resume_timer);
     }
 
     # Return false to stop.
