@@ -1,4 +1,4 @@
-# $Id: Select.pm,v 1.26 2002/05/29 05:24:10 rcaputo Exp $
+# $Id: Select.pm,v 1.27 2002/08/04 18:58:15 rcaputo Exp $
 
 # Select loop substrate for POE::Kernel.
 
@@ -8,7 +8,7 @@ package POE::Kernel::Select;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.26 $ ))[1];
+$VERSION = (qw($Revision: 1.27 $ ))[1];
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -246,17 +246,13 @@ macro substrate_do_timeslice {
   # being watched.  What is a better way to find a 1 bit in any of the
   # vectors?
 
-  my $fileno = 0;
-  @filenos = ();
-  foreach (@kr_filenos) {
-    push(@filenos, $fileno)
-      if ( defined($_) and
-           ( vec($kr_vectors[VEC_RD], $fileno, 1) or
-             vec($kr_vectors[VEC_WR], $fileno, 1) or
-             vec($kr_vectors[VEC_EX], $fileno, 1)
-           )
+  my @filenos = ();
+  foreach (keys %kr_filenos) {
+    push(@filenos, $_)
+      if ( vec($kr_vectors[VEC_RD], $_, 1) or
+           vec($kr_vectors[VEC_WR], $_, 1) or
+           vec($kr_vectors[VEC_EX], $_, 1)
          );
-    $fileno++;
   }
 
   if (TRACE_SELECT) {
