@@ -1,11 +1,12 @@
-# $Id: SocketFactory.pm,v 1.56 2002/06/18 22:13:11 rcaputo Exp $
+# $Id: SocketFactory.pm,v 1.60 2002/08/29 14:48:29 rcaputo Exp $
 
 package POE::Wheel::SocketFactory;
+use POE::Preprocessor ( isa => "POE::Macro::UseBytes" );
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.56 $ ))[1];
+$VERSION = (qw($Revision: 1.60 $ ))[1];
 
 use Carp;
 use Symbol;
@@ -29,6 +30,7 @@ sub MY_MINE_FAILURE    () {  8 }
 sub MY_SOCKET_PROTOCOL () {  9 }
 sub MY_SOCKET_TYPE     () { 10 }
 sub MY_SOCKET_SELECTED () { 11 }
+sub MY_STATE_ERROR     () { 12 }
 
 # Fletch has subclassed SSLSocketFactory from SocketFactory.  He's
 # added new members after MY_SOCKET_SELECTED.  Be sure, if you extend
@@ -46,6 +48,11 @@ BEGIN {
     eval '*F_GETFL       = sub {     0 };';
     eval '*F_SETFL       = sub {     0 };';
   }
+
+  unless (exists $INC{"Socket6.pm"}) {
+    eval "*Socket6::AF_INET6 = sub { ~0 }";
+    eval "*Socket6::PF_INET6 = sub { ~0 }";
+  }
 }
 
 #------------------------------------------------------------------------------
@@ -53,13 +60,16 @@ BEGIN {
 # same operations, it seems, and this is a way to add new ones with a
 # minimum of additional code.
 
-sub DOM_UNIX () { 'unix' }  # UNIX domain socket
-sub DOM_INET () { 'inet' }  # INET domain socket
+sub DOM_UNIX  () { 'unix'  }  # UNIX domain socket
+sub DOM_INET  () { 'inet'  }  # INET domain socket
+sub DOM_INET6 () { 'inet6' }  # INET v6 domain socket
 
 # AF_XYZ and PF_XYZ may be different.
 my %map_family_to_domain =
-  ( AF_UNIX, DOM_UNIX, PF_UNIX, DOM_UNIX,
-    AF_INET, DOM_INET, PF_INET, DOM_INET,
+  ( AF_UNIX,  DOM_UNIX,  PF_UNIX,  DOM_UNIX,
+    AF_INET,  DOM_INET,  PF_INET,  DOM_INET,
+    &Socket6::AF_INET6, DOM_INET6,
+    &Socket6::PF_INET6, DOM_INET6,
   );
 
 sub SVROP_LISTENS () { 'listens' }  # connect/listen sockets
@@ -68,19 +78,25 @@ sub SVROP_NOTHING () { 'nothing' }  # connectionless sockets
 # Map family/protocol pairs to connection or connectionless
 # operations.
 my %supported_protocol =
-  ( DOM_UNIX, { none => SVROP_LISTENS },
-    DOM_INET, { tcp  => SVROP_LISTENS,
-                udp  => SVROP_NOTHING,
-              },
+  ( DOM_UNIX,  { none => SVROP_LISTENS },
+    DOM_INET,  { tcp  => SVROP_LISTENS,
+                 udp  => SVROP_NOTHING,
+               },
+    DOM_INET6, { tcp  => SVROP_LISTENS,
+                 udp  => SVROP_NOTHING,
+               },
   );
 
 # Sane default socket types for each supported protocol.  -><- Maybe
 # this structure can be combined with %supported_protocol?
 my %default_socket_type =
-  ( DOM_UNIX, { none => SOCK_STREAM },
-    DOM_INET, { tcp  => SOCK_STREAM,
-                udp  => SOCK_DGRAM,
-              },
+  ( DOM_UNIX,  { none => SOCK_STREAM },
+    DOM_INET,  { tcp  => SOCK_STREAM,
+                 udp  => SOCK_DGRAM,
+               },
+    DOM_INET6, { tcp  => SOCK_STREAM,
+                 udp  => SOCK_DGRAM,
+               },
   );
 
 #------------------------------------------------------------------------------
@@ -135,6 +151,10 @@ sub _define_accept_state {
           elsif ( $domain eq DOM_INET ) {
             ($peer_port, $peer_addr) = unpack_sockaddr_in($peer);
           }
+          elsif ( $domain eq DOM_INET6 ) {
+            $peer = getpeername($new_socket);
+            ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
+          }
           else {
             die "sanity failure: socket domain == $domain";
           }
@@ -230,6 +250,18 @@ sub _define_connect_state {
           }
         }
 
+        # INET6 socket stacks tend not to.
+        elsif ($domain eq DOM_INET6) {
+          if (defined $peer) {
+            eval {
+              ($peer_port, $peer_addr) = Socket6::unpack_sockaddr_in6($peer);
+            };
+            if (length $@) {
+              $peer_port = $peer_addr = undef;
+            }
+          }
+        }
+
         # What are we doing here?
         else {
           die "sanity failure: socket domain == $domain";
@@ -242,10 +274,40 @@ sub _define_connect_state {
       }
     );
 
+  # Cygwin expects an error state registered to expedite.  This code
+  # is nearly identical the stuff above.
+  $poe_kernel->state
+    ( $self->[MY_STATE_ERROR] = ( ref($self) .
+                                  "($unique_id) -> connect error"
+                                ),
+      sub {
+        # This prevents SEGV in older versions of Perl.
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # Grab some values and stop watching the socket.
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        undef $$socket_selected;
+        $k->select($handle);
+
+        # Throw a failure if the connection failed.
+        $! = unpack('i', getsockopt($handle, SOL_SOCKET, SO_ERROR));
+        if ($!) {
+          (defined $$failure_event) and
+            $k->call( $me, $$failure_event,
+                      'connect', ($!+0), $!, $unique_id
+                    );
+          return;
+        }
+      }
+    );
+
   $self->[MY_SOCKET_SELECTED] = 'yes';
   $poe_kernel->select_write( $self->[MY_SOCKET_HANDLE],
                              $self->[MY_STATE_CONNECT]
                            );
+  $poe_kernel->select_expedite( $self->[MY_SOCKET_HANDLE],
+                                $self->[MY_STATE_ERROR]
+                              );
 }
 
 #------------------------------------------------------------------------------
@@ -303,6 +365,9 @@ sub event {
     $poe_kernel->select_write( $self->[MY_SOCKET_HANDLE],
                                $self->[MY_STATE_CONNECT]
                              );
+    $poe_kernel->select_expedite( $self->[MY_SOCKET_HANDLE],
+                                  $self->[MY_STATE_ERROR]
+                                );
   }
   else {
     die "POE developer error - no state defined";
@@ -380,15 +445,15 @@ sub new {
         undef,                            # MY_SOCKET_PROTOCOL
         undef,                            # MY_SOCKET_TYPE
         undef,                            # MY_SOCKET_SELECTED
+        undef,                            # MY_STATE_ERROR
       ],
       $type
     );
 
   # Default to Internet sockets.
-  $self->[MY_SOCKET_DOMAIN] = ( (defined $params{SocketDomain})
-                                ? $params{SocketDomain}
-                                : AF_INET
-                              );
+  my $domain = delete $params{SocketDomain};
+  $domain = AF_INET unless defined $domain;
+  $self->[MY_SOCKET_DOMAIN] = $domain;
 
   # Abstract the socket domain into something we don't have to keep
   # testing duplicates of.
@@ -419,7 +484,9 @@ sub new {
 
   # Internet sockets use protocols.  Default the INET protocol to tcp,
   # and try to resolve it.
-  elsif ($abstract_domain eq DOM_INET) {
+  elsif ( $abstract_domain eq DOM_INET or
+          $abstract_domain eq DOM_INET6
+        ) {
     my $socket_protocol =
       (defined $params{SocketProtocol}) ? $params{SocketProtocol} : 'tcp';
 
@@ -552,7 +619,6 @@ sub new {
   # context, and translate them into parameters that bind()
   # understands.
   if ($abstract_domain eq DOM_INET) {
-
     # Don't bind if the creator doesn't specify a related parameter.
     if ((defined $params{BindAddress}) or (defined $params{BindPort})) {
 
@@ -562,13 +628,17 @@ sub new {
                         : INADDR_ANY
                       );
 
+      {% use_bytes %}
+
       # Resolve the bind address if it's not already packed.
-      (length($bind_address) == 4)
-        or ($bind_address = inet_aton($bind_address));
+      unless (length($bind_address) == 4) {
+        $bind_address = inet_aton($bind_address);
+      }
+
       unless (defined $bind_address) {
         $! = EADDRNOTAVAIL;
         $poe_kernel->yield( $event_failure,
-                            'inet_aton', $!+0, $!, $self->[MY_UNIQUE_ID]
+                            "inet_aton", $!+0, $!, $self->[MY_UNIQUE_ID]
                           );
         return $self;
       }
@@ -590,7 +660,59 @@ sub new {
       $bind_address = pack_sockaddr_in($bind_port, $bind_address);
       unless (defined $bind_address) {
         $poe_kernel->yield( $event_failure,
-                            'pack_sockaddr_in', $!+0, $!, $self->[MY_UNIQUE_ID]
+                            "pack_sockaddr_in", $!+0, $!, $self->[MY_UNIQUE_ID]
+                          );
+        return $self;
+      }
+    }
+  }
+
+  # Check SocketFactory /Bind.*/ parameters in an Internet socket
+  # context, and translate them into parameters that bind()
+  # understands.
+  elsif ($abstract_domain eq DOM_INET6) {
+
+    # Don't bind if the creator doesn't specify a related parameter.
+    if ((defined $params{BindAddress}) or (defined $params{BindPort})) {
+
+      # Set the bind address, or default to INADDR_ANY.
+      $bind_address = ( (defined $params{BindAddress})
+                        ? $params{BindAddress}
+                        : Socket6::inaddr6_any()
+                      );
+
+      {% use_bytes %}
+
+      # Resolve the bind address.
+      $bind_address =
+        Socket6::gethostbyname2($bind_address, $self->[MY_SOCKET_DOMAIN]);
+      unless (defined $bind_address) {
+        $! = EADDRNOTAVAIL;
+        $poe_kernel->yield( $event_failure,
+                            "gethostbyname2", $!+0, $!, $self->[MY_UNIQUE_ID]
+                          );
+        return $self;
+      }
+
+      # Set the bind port, or default to 0 (any) if none specified.
+      # Resolve it to a number, if at all possible.
+      my $bind_port = (defined $params{BindPort}) ? $params{BindPort} : 0;
+      if ($bind_port =~ /[^0-9]/) {
+        $bind_port = getservbyname($bind_port, $protocol_name);
+        unless (defined $bind_port) {
+          $! = EADDRNOTAVAIL;
+          $poe_kernel->yield( $event_failure,
+                              'getservbyname', $!+0, $!, $self->[MY_UNIQUE_ID]
+                            );
+          return $self;
+        }
+      }
+
+      $bind_address = Socket6::pack_sockaddr_in6($bind_port, $bind_address);
+      unless (defined $bind_address) {
+        $poe_kernel->yield( $event_failure,
+                            "pack_sockaddr_in6", $!+0, $!,
+                            $self->[MY_UNIQUE_ID]
                           );
         return $self;
       }
@@ -652,7 +774,9 @@ sub new {
     # Check SocketFactory /Remote.*/ parameters in an Internet socket
     # context, and translate them into parameters that connect()
     # understands.
-    if ($abstract_domain eq DOM_INET) {
+    if ($abstract_domain eq DOM_INET or
+        $abstract_domain eq DOM_INET6
+       ) {
                                         # connecting if RemoteAddress
       croak 'RemotePort required' unless (defined $params{RemotePort});
       carp 'ListenQueue ignored' if (defined $params{ListenQueue});
@@ -668,20 +792,47 @@ sub new {
         }
       }
 
-      $connect_address = inet_aton($params{RemoteAddress});
+      my $error_tag;
+      if ($abstract_domain eq DOM_INET) {
+        $connect_address = inet_aton($params{RemoteAddress});
+        $error_tag = "inet_aton";
+      }
+      elsif ($abstract_domain eq DOM_INET6) {
+        $connect_address =
+          Socket6::gethostbyname2( $params{RemoteAddress},
+                                   $self->[MY_SOCKET_DOMAIN]
+                                 );
+        $error_tag = "gethostbyname2";
+      }
+      else {
+        die "unknown domain $abstract_domain";
+      }
+
       unless (defined $connect_address) {
         $! = EADDRNOTAVAIL;
         $poe_kernel->yield( $event_failure,
-                            'inet_aton', $!+0, $!, $self->[MY_UNIQUE_ID]
+                            $error_tag, $!+0, $!, $self->[MY_UNIQUE_ID]
                           );
         return $self;
       }
 
-      $connect_address = pack_sockaddr_in($remote_port, $connect_address);
+      if ($abstract_domain eq DOM_INET) {
+        $connect_address = pack_sockaddr_in($remote_port, $connect_address);
+        $error_tag = "pack_sockaddr_in";
+      }
+      elsif ($abstract_domain eq DOM_INET6) {
+        $connect_address =
+          Socket6::pack_sockaddr_in6($remote_port, $connect_address);
+        $error_tag = "pack_sockaddr_in6";
+      }
+      else {
+        die "unknown domain $abstract_domain";
+      }
+
       unless ($connect_address) {
         $! = EADDRNOTAVAIL;
         $poe_kernel->yield( $event_failure,
-                            'pack_sockaddr_in', $!+0, $!, $self->[MY_UNIQUE_ID]
+                            $error_tag, $!+0, $!, $self->[MY_UNIQUE_ID]
                           );
         return $self;
       }
@@ -927,8 +1078,12 @@ call.
 =item SocketDomain
 
 SocketDomain supplies socket() with its DOMAIN parameter.  Supported
-values are AF_UNIX, AF_INET, PF_UNIX and PF_INET.  If SocketDomain is
-omitted, it defaults to AF_INET.
+values are AF_UNIX, AF_INET, AF_INET6, PF_UNIX, PF_INET, and PF_INET6.
+If SocketDomain is omitted, it defaults to AF_INET.
+
+Note: AF_INET6 and PF_INET6 are supplied by the Socket6 module, which
+is available on the CPAN.  You must have Socket6 loaded before
+SocketFactory can create IPv6 sockets.
 
 =item SocketType
 
@@ -1112,7 +1267,7 @@ A sample ErrorEvent handler:
 
 =head1 SEE ALSO
 
-POE::Wheel.
+POE::Wheel, Socket6.
 
 The SEE ALSO section in L<POE> contains a table of contents covering
 the entire POE distribution.
